#!/usr/bin/perl

use FCGI; # for forms input
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Graphics::GnuplotIF;

############################### Globals ########################

# test db access and plotting stuff
my $dsn = "DBI:mysql:host=localhost;database=OTEnv"; # data source name
my $dbh;	# database connection handle
my $sthT;	# statement handle for Temperatures
my $sthB;	# barometer
my $sthH;	# for humidities
my $sthS;	# for stove
my $sthW;	# water
my $tablename = "testing"; # testing for resting, OTData for the real thing

my $tplot; #gnuplot instance

my $time; # "now, in epoch-seconds"
my $data; # from submit post
my ($key,$value); # for the config hash
my %config; # config  Current (only) key is seconds
# array for options box
my @options = (' value="3600">1 hour',' value="86400">1 day',' value="604800">1 week', ' value="2592000">30 days',
' value="5184000">60 days',' value="10368000">120 days',' value="31536000">365 days',' value="63072000">730 days');
 # strings to build option list in HTML
 my %refresh = (3600 => 60, 80400=>60,604800=>756,2592000=>3240,5184000=>6480,10368000=>12960,31536000=>39420,
63072000=>790090);
my $regenerate; # update plots if set
my $request = FCGI::Request; # object wrapping CGI request

#################
sub writecnf
{
 open(CONF, "> /var/tmp/plots.cnf") or die "Couldn't write /var/tmp/plots.cnf:$!\n";
 print CONF "$data\n";
 close CONF;
}

##################
 sub makeoptions
{
 my $line;
 my $dummy = $config{seconds};
# print "dummy:$dummy<br>\n"; 
 print '<SELECT name="seconds" size="8">';
 print "\n";
 foreach $line (@options)
 {
  print '<OPTION'; # start of an option
  if ($line =~ m/"$dummy"/) {$line=~ s/>/ selected>/;} # quotes so  sub-matches won't occur
  print "$line";
  print "</OPTION>\n"; 
 }
 print "</SELECT>\n";
}
##################
sub showplots
{ # make html to show the plots
print <<ENDOFPLOTS;
<div align="left">
<h3>Temperatures</h3>
<img alt="Temperatures" height="400" src="../plots/Temp.png" width="800" />

<h3>Barometer</h3>
<img alt="barometer" height="400" src="../plots/Baro.png" width="800" />

<h3>Relative Humidities</h3>
<img alt="Humidities" height="400" src="../plots/Humid.png" width="800" />

<h3>Water</h3>
<img alt="Water" height="400" src="../plots/Water.png" width="800" />

<h3>Wood stove</h3>
<img alt="Woodstove" height="400" src="../plots/Stove.png" width="800" />

</div>
ENDOFPLOTS
print "<br>\n";
}
##################
sub makeplots
{ # going to be a long one here
 my @raw; # from database
 my $row; # one row of query results
 my (@x,@y1,@y2,@y3,@y4); # data for plots
 my $start = $time - $config{seconds}; # compute how far back from "now" for query

 $dbh = DBI->connect($dsn,"acq","") or die "Cannot connect to db server\n"; # also, some free delay time

 #print "<br>time:$time<br>\n";
 $start = "\'" . "$start" ."\'";
 #print "<br>start:$start<br>\n";

 $sthT = $dbh->prepare("SELECT Epoch, IndoorTemp, OutdoorTemp,BaseTemp,ThermoCJ FROM $tablename WHERE Epoch > $start"); 
 $sthT->execute; 
 @raw = @{$sthT->fetchall_arrayref()}; # get the entire dataset

 foreach $row (@raw) { # separate for plotting
  push @x,$row->[0]; # - 946684800); # convert linux epoch to gnuplot epoch (1-Jan-70 to 1-Jan-2000)? bad dox
  push @y1, $row->[1];
  push @y2, $row->[2];
  push @y3, $row->[3];
  push @y4, $row->[4];
 }
 $tplot = Graphics::GnuplotIF->new(title => "Temperatures", style=>"lines",xlabel=>"Time - UTC",ylabel => "Degrees F") or  die "no plot\n";
 $tplot->gnuplot_cmd ('set term png small size 800,400');
 $tplot->gnuplot_cmd("set output '/var/www/plots/Temp.png'");
 $tplot->gnuplot_cmd ("set grid");
 $tplot->gnuplot_set_plot_titles("Indoor","Outdoor","Basement","ThermoCJ");
 $tplot->gnuplot_cmd("set xdata time");
 $tplot->gnuplot_cmd('set timefmt "%s"'); # picky - %s has to be in double quotes and the whole thing in single...
 $tplot->gnuplot_cmd('set xtics format "%y-%m-%d\n%H:%M"'); # see above
 $tplot->gnuplot_cmd ("set key left bottom");
 $tplot->gnuplot_plot_xy(\@x,\@y1,\@y2,\@y3,\@y4);

#### barometer, Baro.png
 $tplot->gnuplot_cmd("set output '/var/www/plots/Baro.png'");
 $tplot->gnuplot_set_title ("Barometer");
 $tplot->gnuplot_set_ylabel("Air Pressure");
 $tplot->gnuplot_set_plot_titles("hectoPascals");

 
 $sthB = $dbh->prepare("SELECT Barometer FROM $tablename WHERE Epoch > $start"); 
 $sthB->execute; 
 @raw = @{$sthB->fetchall_arrayref()}; # get the entire dataset
 @y1 = (); # zero out Y array
 foreach $row (@raw) { # separate for plotting
  push @y1, $row->[0];
 }
 $tplot->gnuplot_plot_xy(\@x,\@y1);

#### humidity
 $tplot->gnuplot_cmd("set output '/var/www/plots/Humid.png'");
 $tplot->gnuplot_set_title ("Relative Humidity ");
 $tplot->gnuplot_set_ylabel("% Humidity/Barometer");
 $tplot->gnuplot_set_plot_titles("Indoor","Outdoor","Basement");

 $sthH = $dbh->prepare("SELECT IndoorHumid, OutdoorHumid,BaseHumid FROM $tablename WHERE Epoch > $start"); 
 $sthH->execute; 
 @raw = @{$sthH->fetchall_arrayref()}; # get the entire dataset
 @y1 = @y2 = @y3 = (); # zero out Y arrays
 foreach $row (@raw) { # separate for plotting
  push @y1, $row->[0];
  push @y2, $row->[1];
  push @y3, $row->[2];
 }
 $tplot->gnuplot_plot_xy(\@x,\@y1,\@y2,\@y3);
 
# water
 $tplot->gnuplot_cmd("set output '/var/www/plots/Water.png'");
 $tplot->gnuplot_set_title ("Water System");
 $tplot->gnuplot_set_ylabel("Levels/Rates");
 $tplot->gnuplot_set_plot_titles("Level","In Rate/25","Out Rate/25");

 
 $sthH = $dbh->prepare("SELECT WaterLevel, WaterInRate,WaterOutRate FROM $tablename WHERE Epoch > $start"); 
 $sthH->execute; 
 @raw = @{$sthH->fetchall_arrayref()}; # get the entire dataset
 @y1 = @y2 = @y3 = (); # zero out Y arrays
 foreach $row (@raw) { # separate for plotting
  push @y1, $row->[0];
  push @y2, $row->[1]/25;
  push @y3, $row->[2]/25;
 }
 $tplot->gnuplot_plot_xy(\@x,\@y1,\@y2,\@y3);

# Stove.png, whatever is left
 $tplot->gnuplot_cmd("set output '/var/www/plots/Stove.png'");
 $tplot->gnuplot_set_title ("Stove etc");
 $tplot->gnuplot_set_ylabel("Temperatures");

 $tplot->gnuplot_set_plot_titles("Flue","Pi Box","CPU");

 
 $sthH = $dbh->prepare("SELECT Thermocouple, BoxTemp,CPUTemp FROM $tablename WHERE Epoch > $start"); 
 $sthH->execute; 
 @raw = @{$sthH->fetchall_arrayref()}; # get the entire dataset
 @y1 = @y2 = @y3 = (); # zero out Y arrays
 foreach $row (@raw) { # separate for plotting
  push @y1, $row->[0];
  push @y2, $row->[1];
  push @y3, $row->[2];
 }
 $tplot->gnuplot_plot_xy(\@x,\@y1,\@y2,\@y3);
}
##########################
sub chkregen
{ # see if we should regenerate the plots even on normal auto-refresh
 my $lasttime = (stat('/var/www/plots/Temp.png'))[9]; # modify time
 if (($time - $lasttime) > $refresh{$config{seconds}}) {$regenerate = 1;}
}
######### Main ###################################

while ($request->Accept >= 0)
{
 print "Content-type:text/html\n\n"; # get the ball rolling
 print <<EndOfHTML;
<html><head><title>OT Plots</title><meta http-equiv="refresh" content="300"></head>
<body>
<h1>OT Plots</h1> 
EndOfHTML
# magic line for auto-refresh via GET:<meta http-equiv="refresh" content="60"></head>
$time = time(); # now, in epoch seconds

{ # closure for local $/
 local $/;
 $data = <>; # suck up any submit data
}
chomp $data;

if ($data) 
 {
  writecnf(); # user submitted length request
  $regenerate = 1; # make new plots on submit, certainly.
 } else 
 {
  if (-e "/var/tmp/plots.cnf")
  {
   open (CONF, "< /var/tmp/plots.cnf");
   $data = <CONF>;
  } else 
  {  #special case if no cnf file on ramdisk
   $data = "seconds=292000\n"; # get ready and write a .cnf file for later (one week)
   writecnf();
  }
 } 
# have data now for our hash, however we got it
 chomp $data;
($key,$value) = split /=/, $data;
$config{$key} = $value;

# generate/serve plots here
#/var/www/plots ?.png
chkregen(); # see if we should set $regenerate

if ($regenerate) {makeplots();}

showplots();





# create a form with submit and some stuff
 print "<FORM action=\"plots.cgi\" method=\"post\"><br>\n";
 print "Plot Duration<br>\n";
 makeoptions();

 print "<INPUT type=\"submit\" value=\"plot duration\">\n</FORM><br>\n";
 print '<a href="..//index.html">Home</a><br> <p> </p>';
 print "</body></html>\n";
} # end while request
